#version 460 core

#include "/lib/settings.glsl"

#ifdef VOLUMETRIC_LIGHT
#endif

/* RENDERTARGETS: 0 */
layout(location = 0) out vec3 outColor0;

uniform int isEyeInWater, viewHeight, viewWidth, worldTime;
uniform float blindness, far, frameTimeCounter, near, rainStrength;
uniform vec3 fogColor, skyColor;
uniform mat4 gbufferModelViewInverse, gbufferProjectionInverse, shadowModelView, shadowProjection;
uniform sampler2D colortex0, depthtex1, noisetex, shadowtex1;

#ifdef COLORED_SHADOWS
	uniform sampler2D shadowcolor0;
#endif

in vec2 texCoord;

#include "/lib/distort.glsl"

void main() {
	vec3 color = texture(colortex0, texCoord).rgb;

	float depth = texture(depthtex1, texCoord).r;

	if (depth < 1.0) {
		vec3 fogCol = pow(fogColor, vec3(2.4));

		float daylight = min(distance(worldTime, 6000.0), distance(worldTime, 30000.0)) / 12000.0;
		vec3 lightColor = mix(fogCol, max(1.1 - pow(skyColor, vec3(2.4)) - max(daylight - 0.5, 0.0) * 4.0, 0.0), min(daylight + 0.1, 1.0 - rainStrength * 0.9));

		if (length(lightColor) * (10.0 + SUNLIGHT - AMBIENT) > 5.0) {
			color = pow(color, vec3(2.4));

			float ray = 0.0;

			const int noiseTextureResolution = 256;
			float random = texture(noisetex, texCoord * 100.0 + frameTimeCounter * 0.01).r;

			vec4 NDCPos = vec4(2.0 * texCoord - 1.0, 0.0, 1.0);

			float linearDepth = (near * far) / (depth * (near - far) + far);

			//uint samples = uint(min(linearDepth * 4.0, 20.0));

			for (uint i = 1; i <= VL_SAMPLES; i++) {
				float sampleDepth = mix(depth, 0.0, random * log(0.1 * i / VL_SAMPLES + 1.0));
				//float sampleDepth = depth * (1.0 - random * i * 0.0001);
				//float sampleDepth = depth * (1.0 - random * i / samples * 5.0);

				NDCPos.z = sampleDepth * 2.0 - 1.0;
				vec4 viewPos = gbufferProjectionInverse * NDCPos;
				viewPos /= viewPos.w;
				vec4 worldPos = gbufferModelViewInverse * viewPos;
				vec3 shadowPos = distort((shadowProjection * shadowModelView * worldPos).xyz) * 0.5 + 0.5;

				ray += step(shadowPos.z, texture(shadowtex1, shadowPos.xy).r);
			}

			color += ray * 0.000025 * mix(vec3(1.0), fogCol, isEyeInWater) * lightColor * SUNLIGHT * VL_STRENGTH / VL_SAMPLES;

			color = pow(color, vec3(1.0 / 2.4));
		}
	}

	outColor0 = color;
}