#version 460 core

#include "/lib/settings.glsl"

uniform int worldTime;
uniform float blindness, far, rainStrength;
uniform vec3 fogColor, shadowLightPosition, skyColor;
uniform mat3 normalMatrix;
uniform mat4 modelViewMatrix, projectionMatrix;

in vec3 vaNormal, vaPosition;
in vec4 vaColor;

out vec4 tintColor;

void main() {
	vec4 viewPos = modelViewMatrix * vec4(vaPosition, 1.0);

	float dist = length(viewPos.xyz * 0.6);
	float fog = pow(dist / far, 10.0 / (5.0 + FOG + blindness * 5000.0 + rainStrength * RAIN_FOG));

	if (fog < 0.999) {
		vec3 fogCol = pow(fogColor, vec3(2.4));

		float diffuse = max(dot(normalize(normalMatrix * vaNormal), normalize(shadowLightPosition)), 0.5);

		float daylight = min(distance(worldTime, 6000.0), distance(worldTime, 30000.0)) / 12000.0;
		vec3 sunlight = 0.5 * mix(fogCol, max(1.1 - pow(skyColor, vec3(2.4)) - max(daylight - 0.5, 0.0) * 4.0, 0.0), min(daylight + 0.1, 1.0));

		tintColor = vec4(pow((pow(vaColor.rgb, vec3(2.4)) + sunlight * min(diffuse, float(sunlight))) * 0.5 + fogCol, vec3(1.0 / 2.4)), vaColor.a * min(dist * 0.01, 1.0 - fog));
	} else {
		tintColor = vec4(0.0);
	}

	gl_Position = projectionMatrix * viewPos;
}