#version 460 core

#include "/lib/settings.glsl"
#include "/lib/lightmap.glsl"

uniform int heldBlockLightValue, heldBlockLightValue2, heldItemId, heldItemId2;
uniform vec3 shadowLightPosition;
uniform mat3 normalMatrix;
uniform mat4 projectionMatrix;

#if !defined END && !defined NETHER 
	uniform int worldTime;
	uniform vec3 fogColor, skyColor;
#endif

in vec2 vaUV0;
in vec3 vaNormal, vaPosition;
in vec4 vaColor;

out vec2 texCoord;
out vec3 tintColor;

void main() {
	gl_Position = projectionMatrix * vec4(vaPosition, 1.0);
	texCoord = vaUV0;

	float diffuse = dot(normalize(normalMatrix * vaNormal), normalize(shadowLightPosition));

	#if defined END || defined NETHER
		tintColor = vaColor.rgb * mix(max(diffuse, 0.5), 1.0, 0.5);
	#else
		float daylight = min(distance(worldTime, 6000.0), distance(worldTime, 30000.0)) / 12000.0;
		vec3 lightColor = mix(pow(fogColor, vec3(2.4)), max(1.1 - pow(skyColor, vec3(2.4)) - max(daylight - 0.5, 0.0) * 4.0, 0.0), min(daylight + 0.1, 1.0));

		const float ambient = AMBIENT * 0.05;

		tintColor = pow(vaColor.rgb, vec3(2.4));

		if (diffuse > 0.0) {
			vec3 sunlight = lightColor * SUNLIGHT * 0.1;

			tintColor.rgb *= sunlight * min(diffuse, float(sunlight)) + ambient;
		} else {
			tintColor.rgb *= ambient;
		}

		tintColor = pow(tintColor, vec3(1.0 / 2.4));
	#endif

	#if EMISSION
		const vec3 maxLight = vec3(EMISSION * 0.1);
	#else
		vec3 maxLight = texture(lightmap, vec2(1.0, -1.0)).rgb;
	#endif

	vec3 light = mix(texture(lightmap, lmCoord).rgb, maxLight, min((heldBlockLightValue + heldBlockLightValue2) / 15.0 + heldItemId + heldItemId2 + 2.0, 1.0));

	#ifdef NETHER
		tintColor *= mix(vec3(1.0, 0.6, 0.76) * AMBIENT * 0.1, light * 1.5, float(light) - 0.75);
	#else
		#ifdef END
			tintColor *= mix(vec3(0.76, 0.4, 0.76) * AMBIENT * 0.1, light * 1.5, float(light) - 0.75);
		#else
			tintColor *= light;
		#endif
	#endif
}