#version 460 core

#include "/lib/settings.glsl"
#include "/lib/lightmap.glsl"

#ifdef GLOWING_REDSTONE_BLOCK
#endif
#ifdef GLOWING_EMERALD_BLOCK
#endif
#ifdef GLOWING_LAPIS_BLOCK
#endif
#ifdef GLOWING_FLOWERS
#endif
#ifdef GLOWING_FUNGI
#endif

uniform int isEyeInWater;
uniform float blindness, far, rainStrength;
uniform vec3 chunkOffset, shadowLightPosition, cameraPosition;
uniform mat3 normalMatrix;
uniform mat4 modelViewMatrix, projectionMatrix;

#ifdef BLOCK_ENTITY
	uniform mat4 gbufferModelViewInverse;
#endif

#if defined TRANSPARENT && WAVES != 0
	uniform float frameTimeCounter;
#endif

in vec2 vaUV0;
in vec3 mc_Entity, vaNormal, vaPosition;
in vec4 vaColor;

out float depth, diffuse, fog;
out vec2 texCoord;
out vec4 tintColor;

#ifdef SHADOWS
	uniform mat4 shadowModelView, shadowProjection;
	
	out vec3 shadowPos;

	#include "/lib/distort.glsl"
#endif

void main() {
	texCoord = vaUV0;
	tintColor = vaColor;

	vec3 pos = vaPosition + chunkOffset;

	#ifdef TRANSPARENT
		if (mc_Entity.r == 1) {
			tintColor.a *= WATER_OPACITY * 0.01;

			#if WAVES
				float time = frameTimeCounter * WAVE_SPEED;
				pos.y += (sin(cameraPosition.x + pos.x + time) * sin(cameraPosition.z + pos.z + time) - 1.0) * WAVES * 0.0375;
			#endif
		}
	#endif

	diffuse = dot(normalize(normalMatrix * vaNormal), normalize(shadowLightPosition));

	#ifdef SHADOWS
		#ifdef BLOCK_ENTITY
			vec4 viewPos = modelViewMatrix * vec4(pos, 1.0);
			shadowPos = distort((shadowProjection * shadowModelView * gbufferModelViewInverse * viewPos).xyz) * 0.5 + 0.5;
		#else
			vec4 worldPos = vec4(pos, 1.0);
			vec4 viewPos = modelViewMatrix * worldPos;
			shadowPos = distort((shadowProjection * shadowModelView * worldPos).xyz) * 0.5 + 0.5;
		#endif
	#else
		vec4 viewPos = modelViewMatrix * vec4(pos, 1.0);

		diffuse = max(diffuse, 0.0);
	#endif

	fog = min(pow(length(viewPos.xyz) / far, 10.0 / (FOG + blindness * 5000.0 + rainStrength * RAIN_FOG + pow(isEyeInWater, 4.0) * WATER_FOG)), 1.0);

	#if EMISSION
		const vec3 maxLight = vec3(EMISSION * 0.1);
	#else
		vec3 maxLight = texture(lightmap, vec2(1.0, -1.0)).rgb;
	#endif

	vec3 light = mix(texture(lightmap, lmCoord).rgb, maxLight, clamp(mc_Entity.r - 1.0, 0.0, 1.0));

	#ifdef END
		light = mix(vec3(0.76, 0.4, 0.76) * AMBIENT * 0.05, light * 1.5, float(light) - 0.75);
	#endif

	tintColor.rgb *= light;

	vec4 position = projectionMatrix * viewPos;

	depth = position.z / position.w;
	gl_Position = position;
}