#version 460 core

#include "/lib/settings.glsl"
#include "/lib/lightmap.glsl"

uniform int isEyeInWater;
uniform float blindness, far;
uniform vec3 chunkOffset;
uniform mat4 modelViewMatrix, projectionMatrix;

in vec2 vaUV0;
in vec3 mc_Entity, vaPosition;
in vec4 vaColor;

out float fog;
out vec2 texCoord;
out vec3 tintColor;

void main() {
	texCoord = vaUV0;

	vec4 viewPos = modelViewMatrix * vec4(vaPosition + chunkOffset, 1.0);

	fog = min(pow(length(viewPos.xyz) / far, 10.0 / (FOG + blindness * 5000.0 + pow(isEyeInWater, 4.0) * WATER_FOG)), 1.0);

	gl_Position = projectionMatrix * viewPos;

	#if EMISSION
		const vec3 maxLight = vec3(EMISSION * 0.1);
	#else
		vec3 maxLight = texture(lightmap, vec2(1.0, -1.0)).rgb;
	#endif

	vec3 light = mix(texture(lightmap, lmCoord).rgb, maxLight, clamp(mc_Entity.r - 1.0, 0.0, 1.0));

	tintColor = vaColor.rgb * mix(vec3(1.0, 0.6, 0.76) * AMBIENT * 0.05, light * 1.5, float(light) - 0.75);
}